@@ -95,7 +95,7 @@
 #![doc(
     html_logo_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
     html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
-    html_root_url = "https://docs.rs/pem-rfc7468/0.1.1"
+    html_root_url = "https://docs.rs/pem-rfc7468/0.2.1"
 )]
 #![forbid(unsafe_code, clippy::unwrap_used)]
 #![warn(missing_docs, rust_2018_idioms, unused_qualifications)]
@@ -112,7 +112,7 @@ mod error;
 mod grammar;
 
 pub use crate::{
-    decoder::decode,
+    decoder::{decode, decode_label},
     encoder::{encode, encoded_len, LineEnding},
     error::{Error, Result},
 };
